module eml_io

  use param  
  use dal
  use errorHandling
  use emldata
  use edetect_io

  implicit none

  private

  public get_sources, get_smaps, get_event, co_image, co_bgima, co_expima, &
         mulmlpari, wr_stab, wr_srcmap, mlout

contains

  !---------------------------------------------------------------------
  subroutine get_sources (nsou, xx, yy, sn, src, in)
  !---------------------------------------------------------------------

    real, dimension(max_sou), intent(out) :: xx, yy, sn     
    real, dimension(max_ima,max_sou), intent(out) :: src
    integer, dimension(max_sou), intent(out) :: in
    integer, intent(out) :: nsou

    integer  :: l1,nrow,inst,band,isou
    integer  :: ipoint,snrmin,ierr
    real     :: xms,yms,snr,scts

    integer(kind=int32) :: i, is, ins, prio

    real(kind=single), dimension(:), pointer, save :: rp4, rp5, rp6, rp7, rp8
    integer(kind=int32), dimension(:), pointer, save ::ip1, ip2, ip3

    type(DataSetT) :: set
    type(TableT) :: tab
    type(ColumnT) :: col(8)

    ierr=0
      
! should be read from parameter file
    snrmin = 1

! open list

    set = DataSet(c_d1list, read) 
    tab = table(set, "SRCLIST")  

    l1=1
    col(l1) = column(tab,"BOX_ID_SRC",read)  
    ip1 => int32Data(col(l1))
    l1=l1+1 
    col(l1) = column(tab,"ID_INST",read)  
    ip2 => int32Data(col(l1))
    l1=l1+1 
    col(l1) = column(tab,"ID_BAND",read)  
    ip3 => int32Data(col(l1))
    l1=l1+1 
    col(l1) = column(tab,"X_IMA",read)
    rp4 => real32Data(col(l1))
    l1=l1+1 
    col(l1) = column(tab,"Y_IMA",read)
    rp5 => real32Data(col(l1))
    l1=l1+1
    col(l1) = column(tab,"SCTS",read)
    rp6 => real32Data(col(l1))
    l1=l1+1
    col(l1) = column(tab,"BG_MAP",read)
    rp7 => real32Data(col(l1))
    l1=l1+1
    col(l1) = column(tab,"SIGMA",read)
    rp8 => real32Data(col(l1))

    nrow = numberOfRows(tab)

    is = 0
    prio = 2
    do i=0,nrow-1

      isou = ip1(i)
      inst = ip2(i)
      band = ip3(i)
      xms = rp4(i)
      yms = rp5(i)
      scts = rp6(i)
      snr = rp8(i)

      if (i == 0) then
        is = is+1
        prio = 2
      else if (ip1(i) /= ip1(i-1)) then
        if (sn(is) >= snrmin) is = is+1
        prio = 2
      end if  

      if ((inst == 0 .or. inst == 7 .or. &
           band == 0 .or. band == 7) .and. prio > 0 .or. &
          prio > 1) then
        in(is) = isou
        xx(is) = xms
        yy(is) = yms
        sn(is) = snr
      end if

      if (inst == 0 .or. inst == 7) prio = 0
      if (band == 0 .or. band == 7) prio = 1
      if (inst /= 0 .and. band /= 0 .and. &
          inst /= 7 .and. band /= 7) then
        do ins=1,num_instr
          if (inst == expip(ins)) then
            ipoint = ip(ins, band)
            src(ipoint, is) = scts
          end if
        end do      
      end if

    end do

    nsou=is

    write(errstr,*) nsou, " sources read in"
    call message(AppMsg,Verbose)

  end subroutine get_sources

  !---------------------------------------------------------------------
  subroutine get_smaps (no,x_start,y_start,x_end,y_end)
  !---------------------------------------------------------------------
      
    integer         i,no,numsm,maxsm
    parameter       (maxsm=1000)
    integer         stasm(maxsm),endsm(maxsm)
    integer         listsm(20000),stasmo(20000)
    integer         x_start, x_end, y_start, y_end

    common /smaps/ numsm,stasm,endsm

! meaning of nosm ?????
!    call smoke (no,'b',x_start,y_start,x_end,y_end,0,0,0,0, &
!                numsm,listsm,nosm,stasmo,ierr)

! achtung: was passiert, wenn nosm.eq.listsm(numsm) ???
    do i=1,numsm
      stasm(i)=stasmo(listsm(i))
      endsm(i)=stasmo(listsm(i)+1)-1
    end do

  end subroutine get_smaps
      
  !---------------------------------------------------------------------
  subroutine get_event (no,ixs,iys,ixd,iyd,iamp,end_ev)
  !---------------------------------------------------------------------

    integer      no,nolast,numsm,maxsm,iseq,imap
    parameter    (maxsm=1000)
    integer      stasm(maxsm),endsm(maxsm)
    integer      ixs,iys,ixd,iyd,iamp
!    integer      icol1,icol2,icol3,icol4,icol5
    logical      end_ev 

    common /smaps/ numsm,stasm,endsm

    data nolast /-99/ 

    save

    if (no.ne.nolast) then
      imap = 1
      iseq = stasm(imap)
      nolast = no
    end if       

    iseq = iseq+1
    if (iseq.gt.endsm(imap)) then
      imap = imap+1
      iseq=stasm(imap)
    end if

    end_ev=.false.
    if (iseq.gt.endsm(numsm)) then
      write (6,*) 'end of event file reached'
      end_ev=.true. 
    end if

  end subroutine get_event

  !---------------------------------------------------------------------
  subroutine mulmlpari 
  !---------------------------------------------------------------------

    integer :: i 
    character(len=1024) :: string
    character(len=16), dimension(max_ima) :: instrume 
    !real, dimension(max_ima) :: pimax, pimin
    integer, dimension(max_ima) :: band
    integer :: necf, nxidecf, npi

    type(datasett) :: set
    type(tablet) :: tab
    
    c_d1list = stringParameter("boxlistset")
    outtab = stringParameter("mllistset")
    dmlmin = realParameter("mlmin")       
!    dmlsou = realParameter("dmlsoumin")
    dmlext = realParameter("dmlextmin")
    max_sfit = intParameter("nmaxfit")
    phofla = booleanParameter("useevents")
    if (phofla) call error("IllegalFlag","Photon mode not yet implemented")    
    posfla = booleanParameter("fitposition")
    extfla = booleanParameter("fitextent")
!    if (extfla) then
!      call warning("NoExtent","Extent fitting is disabled in this version")
!      extfla=.false.
!    end if
    expfla = booleanParameter("withexpimage")
    cntfla = booleanParameter("fitcounts")       
    errfla = booleanParameter("determineerrors")
    xidfla = booleanParameter("withxidband")
    hotfla = booleanParameter("withhotpixelfilter")
    usecalpsf = booleanParameter("usecalpsf")
    ences  = realParameter("scut")
    encee  = realParameter("ecut") 
    do i=1, parameterCount("hrdef")
      hrdef(i) = intParameter("hrdef",i-1)
    end do
    do i=1,2
      xiddef(i)= intParameter("xiddef",i-1)
    end do

    pixim = 3600.0 * dabs(crdelt1) 

    xoff=0.
    yoff=0.
    if (booleanParameter("withoffsets")) then
      set = dataSet(stringParameter("mergedListset"), READ)
      if (hasAttribute(set,"XOFFPNM1")) then    
        xoff(2)=Real32Attribute(set,"XOFFPNM1")
      else 
        tab = table(set,"SRCLIST")
        if (hasAttribute(tab,"XOFFPNM1")) then
          xoff(2)=Real32Attribute(tab,"XOFFPNM1")
        end if        
      end if
      if (hasAttribute(set,"XOFFPNM2")) then    
        xoff(3)=Real32Attribute(set,"XOFFPNM2")
      else 
        tab = table(set,"SRCLIST")
        if (hasAttribute(tab,"XOFFPNM2")) then
          xoff(3)=Real32Attribute(tab,"XOFFPNM2")
        end if
      end if
      if (hasAttribute(set,"YOFFPNM1")) then    
        yoff(2)=Real32Attribute(set,"YOFFPNM1")
      else 
        tab = table(set,"SRCLIST")
        if (hasAttribute(tab,"YOFFPNM1")) then
          yoff(2)=Real32Attribute(tab,"YOFFPNM1")
        end if
      end if
      if (hasAttribute(set,"YOFFPNM2")) then    
        yoff(3)=Real32Attribute(set,"XOFFPNM2")
      else 
        tab = table(set,"SRCLIST")
        if (hasAttribute(tab,"YOFFPNM2")) then
          yoff(3)=Real32Attribute(tab,"YOFFPNM2")
        end if
      end if
      call release(set)
    end if

    if (phofla) then
      call get_files("bkgimagesets",c_bgima,instrume,band,.true.)
      if (expfla) call get_files("expimagesets",c_expima,instrume,band,.false.)
      call get_files("imagesets",c_image,instrume,band,.false.)
    else
      call get_files("imagesets",c_image,instrume,band,.true.)
      call get_files("bkgimagesets",c_bgima,instrume,band,.false.)
      if (expfla) call get_files("expimagesets",c_expima,instrume,band,.false.)
    end if

!    do i=1, parameterCount("pointresponse")
!      point_resp(i) = Parameter("pointresponse",i-1)
!    end do
 
    npi=parameterCount("pimin")
    pimin=2000
    do i=1,npi 
      pimin(i)=intParameter("pimin",i-1)
    end do
    if (npi/=nim) call warning("FileMismatch", &
      "Number of PI boundaries not equal number of images") 

    npi=parameterCount("pimin")
    pimax=4500
    do i=1,npi 
      pimax(i)=intParameter("pimax",i-1)
    end do
    if (npi/=nim) call warning("FileMismatch", &
      "Number of PI boundaries not equal number of images") 
    
    do i=1,nim
      energy(i)=(pimin(i)+pimax(i))/2.0
    end do    

    necf=parameterCount("ecf")
    ecf=1.0
    do i=1, necf
      ecf(i) = realParameter("ecf",i-1)
    end do
    if (necf/=nim) call warning("FileMismatch", &
      "Number of ECFs not equal number of images") 

    if (xidfla) then
      nxidecf=parameterCount("xidecf")
      do i=1, nxidecf
        xidecf(i) = realParameter("xidecf",i-1)
      end do
      if (nxidecf/=num_instr) call warning("fileMismatch", &
        "Number of XID band ECFs not equal to number of instruments")
    end if 
    
    if (usecalpsf) then
      do i=1,num_instr
        point_resp(i) = expip(i)
      end do
    else
      point_resp=4
    end if

  end subroutine mulmlpari

  !---------------------------------------------------------------------
  subroutine wr_stab(ll,nsrc,insrc,id_cluster,xms,yms,scts,ra,dec,lii,bii, &
                     exms,eyms,escts,eradec,exi_ml,ext,ext_err,ext_ml,&
                     ecut2,flags) 
  !---------------------------------------------------------------------

    integer, intent(in) :: nsrc
    integer, dimension(max_fit), intent(IN) :: insrc
    integer, intent(IN) :: id_cluster 
    real, intent(in) :: ecut2
    real, dimension(max_fit), intent(in) :: xms, yms, exms, eyms
    real, dimension(max_fit) :: ext, ext_err, ext_ml 
    real, dimension(max_ima,max_fit), intent(in) :: scts, escts, exi_ml  
    character(len=*), intent(in) :: flags 
    real(kind=double), dimension(max_fit), intent(in) :: ra,dec,lii,bii
    real(kind=single), dimension(max_fit), intent(in) :: eradec 
    real, dimension(max_ima,max_fit) :: bkg, expos, flx, rate, eflx, erate

    real, dimension(max_ins,max_fit) :: sctsi, esctsi, flxi, eflxi, &
                                        ratei, eratei, bkgi, expoi, exi_mli  
    real, dimension(:,:,:), allocatable :: scts0, escts0, &
                                flx0, eflx0, rate0, erate0, expos0, &
                                bkg0, exi_ml0, ext_ml0
    real, dimension(max_fit) :: sctst, esctst, flxt, eflxt, ratet, &
                                eratet, bkgt, expot, exi_mlt
    real(kind=single), dimension(:,:), allocatable :: hr1, hr2, hr3, &
                                                     hr1_err, hr2_err, hr3_err
    real(kind=single) :: sexp, cutrad, hew, fac, ppt, ppi(max_ins), dum
    integer :: num_instr0, num_image0(max_ima), xid_arr0(max_ima)
    integer :: numt, ifac, nflux
    integer :: nexp,ix0,iy0,ix,iy,i,i0,j,j0,jj,k,l,ll,ipoint,instused,num_ima 
    integer, save :: id_src = 0
    logical, dimension(0:4) :: instflg

! Determine rates, fluxes, sums, per band and total

    allocate (scts0(max_ins,max_bands,nsrc))
    allocate (escts0(max_ins,max_bands,nsrc))
    allocate (flx0(max_ins,max_bands,nsrc))
    allocate (eflx0(max_ins,max_bands,nsrc))
    allocate (rate0(max_ins,max_bands,nsrc))
    allocate (erate0(max_ins,max_bands,nsrc))
    allocate (expos0(max_ins,max_bands,nsrc))
    allocate (bkg0(max_ins,max_bands,nsrc))
    allocate (exi_ml0(max_ins,max_bands,nsrc))
    allocate (ext_ml0(max_ins,max_bands,nsrc))

    allocate (hr1(max_ins,nsrc))
    allocate (hr2(max_ins,nsrc))
    allocate (hr3(max_ins,nsrc))
    allocate (hr1_err(max_ins,nsrc))
    allocate (hr2_err(max_ins,nsrc))
    allocate (hr3_err(max_ins,nsrc))

    numt=0
    do i=1,num_instr
      fac=1.0
      do ifac=2,num_image(i)
        fac=fac*ifac     
      end do
      ppi(i)=log(fac)
      numt = numt+num_image(i)
    end do
    fac=1.0
    do ifac=2,numt
      fac=fac*ifac
    end do
    ppt=log(fac)       
!    sctsi(1:num_instr,1:nsrc)=0.
!    esctsi(1:num_instr,1:nsrc)=0.
!    flxi(1:num_instr,1:nsrc)=0.
!    eflxi(1:num_instr,1:nsrc)=0.
!    ratei(1:num_instr,1:nsrc)=0.
!    eratei(1:num_instr,1:nsrc)=0.
!    exi_mli(1:num_instr,1:nsrc)=0.
    sctsi=0.
    esctsi=0.
    flxi=0.
    eflxi=0.
    ratei=0.
    eratei=0.
!    do i=1,num_instr
!      exi_mli(i,1:nsrc)=-ppi(i)
!    end do
    exi_mli=0.
    exi_mlt(1:nsrc)=0. ! -ppt
    expos=1.0
!    expoi(1:num_instr,1:nsrc)=0.
!    expoi=0.
!    expot(1:nsrc)=0.
!   bkgi(1:num_instr,1:nsrc)=0.
    bkgi=0.
    bkgt(1:nsrc)=0.
    sctst(1:nsrc)=0.
    esctst(1:nsrc)=0.
    flxt(1:nsrc)=0.
    eflxt(1:nsrc)=0.
    ratet(1:nsrc)=0.
    eratet(1:nsrc)=0.
    scts0=0.
    escts0=0.
    flx0=0.
    eflx0=0.
    rate0=0.
    erate0=0.
    exi_ml0=0.
    ext_ml0=0.
    bkg0=0.
    expos0=0.
    instflg(0)=.true.
    instflg(1:4)=.false.
    num_instr0=0
    hr1=-999.
    hr1_err=-999.
    hr2=-999.
    hr2_err=-999.
    hr3=-999.
    hr3_err=-999.
    xid_arr0=-1

    do i=1,nsrc
      do j0=1,num_instr
        j=expip(j0)
        instflg (j)=.true.
        num_instr0=max(j,num_instr0)
        num_image0(j)=num_image(j0)
        xid_arr0(j)=xid_arr(j0)

        do l=1,num_image(j0)
          ipoint=ip(j0,l) 

          ix0 = nint(xms(i)+0.5) ! max(int(xms(i)),1)
          iy0 = nint(yms(i)+0.5) ! max(int(yms(i)),1)
          if (expfla) then
            expos(ipoint,i) = exp_image(ipoint,ix0,iy0)
            if (expos(ipoint,i) <= 0.0) then
              sexp=0.0
              nexp = 0
              do ix=ix0-2,ix0+2
                do iy=iy0-2,iy0+2
                  if (exp_image(ipoint,ix,iy) > 0.0) then
                    sexp=sexp+exp_image(ipoint,ix,iy)
                    nexp=nexp+1
                  end if
                end do
              end do
              expos(ipoint,i) = sexp/nexp
            end if
          end if
          bkg(ipoint,i) = bg_image0(ipoint,ix0,iy0) 
          flx(ipoint,i)  = scts(ipoint,i)/expos(ipoint,i)/ecf(ipoint)*1.e-11
          eflx(ipoint,i) = escts(ipoint,i)/expos(ipoint,i)/ecf(ipoint)*1.e-11
          rate(ipoint,i) = scts(ipoint,i)/expos(ipoint,i)
          erate(ipoint,i)= escts(ipoint,i)/expos(ipoint,i)
          scts0(j,l,i) = scts0(j,l,i)+scts(ipoint,i)
          escts0(j,l,i) = escts0(j,l,i)+escts(ipoint,i)**2
          flx0(j,l,i) = flx0(j,l,i)+flx(ipoint,i)
          eflx0(j,l,i) = eflx0(j,l,i)+eflx(ipoint,i)**2
          rate0(j,l,i) = rate0(j,l,i)+rate(ipoint,i)
          erate0(j,l,i) = erate0(j,l,i)+erate(ipoint,i)**2
          exi_ml0(j,l,i) = exi_ml0(j,l,i)+exi_ml(ipoint,i)
          bkg0(j,l,i) = bkg0(j,l,i)+bkg(ipoint,i)
          expos0(j,l,i) = expos0(j,l,i)+expos(ipoint,i)
        end do
          
        !if (xid_arr(j0)>0) then
        !  print *,j0,num_image(j0),xid_arr(j0),xidecf(1:4) 
        !  ipoint=xid_arr(j0) 
        !  flx(ipoint,i)  = scts(ipoint,i)/expos(ipoint,i)/ &
        !                   xidecf(j0)*1.e-11
        !  eflx(ipoint,i) = escts(ipoint,i)/expos(ipoint,i)/ &
        !                   xidecf(j0)*1.e-11
        !  flx0(j,num_image(j0),i)=flx0(j,num_image(j0),i)+flx(ipoint,i)
        !  eflx0(j,num_image(j0),i)=eflx0(j,num_image(j0),i)+eflx(ipoint,i)**2
        !  print *, flx0(j,num_image(j0),i),flx(ipoint,i)
        !end if

        num_ima=num_image(j0)
        if (xid_arr(j0) > 0) num_ima=num_image(j0)-1        
        do l=1,num_ima
          ipoint=ip(j0,l) 

!         expoi(j,i)=expoi(j,i)+expos(ipoint,i)
!         expot(i)=expot(i)+expos(ipoint,i)
          bkgi(j,i)=bkgi(j,i)+bkg(ipoint,i)
          bkgt(i)=bkgt(i)+bkg(ipoint,i)
          exi_mli(j,i)=exi_mli(j,i)+exi_ml(ipoint,i)
          exi_mlt(i)=exi_mlt(i)+exi_ml(ipoint,i) 
           
          sctsi(j,i)=sctsi(j,i)+scts(ipoint,i) 
          esctsi(j,i)=esctsi(j,i)+escts(ipoint,i)**2

          flxi(j,i)=flxi(j,i)+flx(ipoint,i)
          eflxi(j,i)=eflxi(j,i)+eflx(ipoint,i)**2
          ratei(j,i)=ratei(j,i)+rate(ipoint,i)
          eratei(j,i)=eratei(j,i)+erate(ipoint,i)**2

          sctst(i)=sctst(i)+scts(ipoint,i)
          esctst(i)=esctst(i)+escts(ipoint,i)**2

!         flxt(i)=flxt(i)+flx(ipoint,i)
!         eflxt(i)=eflxt(i)+eflx(ipoint,i)**2
          ratet(i)=ratet(i)+rate(ipoint,i)
          eratet(i)=eratet(i)+erate(ipoint,i)**2
        end do

        esctsi(j,i)=sqrt(esctsi(j,i))
        eflxi(j,i)=sqrt(eflxi(j,i))
        eratei(j,i)=sqrt(eratei(j,i))
!       expoi(j,i)=expoi(j,i)/num_image0(j) ????

      end do
          
      nflux=0
      do j0=1,num_instr
        j=expip(j0)
        if (flxi(j,i) > 0) then
          flxt(i) = flxt(i)+flxi(j,i)
          eflxt(i)= eflxt(i)+eflxi(j,i)**2 
          nflux=nflux+1
        end if   
        do l=1,num_image(j0)
          escts0(j,l,i) = sqrt(escts0(j,l,i))
          eflx0(j,l,i) = sqrt(eflx0(j,l,i))
          erate0(j,l,i) = sqrt(erate0(j,l,i))
        end do
!        expot(i)=expot(i)+expoi(j,i)
      end do
      flxt(i)=flxt(i)/nflux
      eflxt(i)=eflxt(i)**0.5/nflux

      esctst(i)=sqrt(esctst(i))
!      eflxt(i)=sqrt(eflxt(i))
      eratet(i)=sqrt(eratet(i))
    end do

    cutrad = sqrt(ecut2) 

!    do j=1,num_instr0
!     if (instflg(j)) then
    do j0=1,num_instr
     j=expip(j0)
     do i=1,nsrc
      if (hrdef(1)>0.and.hrdef(2)>0.and. &
          max(hrdef(1),hrdef(2))<=num_image(j0)) then    
       hr1(j,i)=(max(0.,rate0(j,hrdef(2),i))-max(0.,rate0(j,hrdef(1),i)))/ &
                (max(0.,rate0(j,hrdef(2),i))+max(0.,rate0(j,hrdef(1),i)))
       hr1_err(j,i)=2.*sqrt((rate0(j,hrdef(1),i)*erate0(j,hrdef(2),i))**2+ &
                            (rate0(j,hrdef(2),i)*erate0(j,hrdef(1),i))**2)/&
                (max(0.,rate0(j,hrdef(2),i))+max(0.,rate0(j,hrdef(1),i)))**2
      end if
      if (hrdef(3)>0.and.hrdef(4)>0.and. &
          max(hrdef(3),hrdef(4))<=num_image(j0)) then    
       hr2(j,i)=(max(0.,rate0(j,hrdef(4),i))-max(0.,rate0(j,hrdef(3),i)))/ &
                (max(0.,rate0(j,hrdef(4),i))+max(0.,rate0(j,hrdef(3),i))) 
       hr2_err(j,i)=2.*sqrt((rate0(j,hrdef(3),i)*erate0(j,hrdef(4),i))**2+ &
                            (rate0(j,hrdef(4),i)*erate0(j,hrdef(3),i))**2)/&
                (max(0.,rate0(j,hrdef(4),i))+max(0.,rate0(j,hrdef(3),i)))**2
      end if
      if (hrdef(5)>0.and.hrdef(5)>0.and. &
          max(hrdef(5),hrdef(6))<=num_image(j0)) then    
       hr3(j,i)=(max(0.,rate0(j,hrdef(6),i))-max(0.,rate0(j,hrdef(5),i)))/ &
                (max(0.,rate0(j,hrdef(6),i))+max(0.,rate0(j,hrdef(5),i)))
       hr3_err(j,i)=2.*sqrt((rate0(j,hrdef(5),i)*erate0(j,hrdef(6),i))**2+ &
                            (rate0(j,hrdef(6),i)*erate0(j,hrdef(5),i))**2)/&
                (max(0.,rate0(j,hrdef(6),i))+max(0.,rate0(j,hrdef(5),i)))**2
      end if
     end do
    end do
!     end if
!    end do

    hew = 0. 
 
! Write out results
    instused=0
    do i=1,4
      if (instflg(i)) instused=instused+1
    end do

! Instrument summary only required if more than one instrument
    if (instused > 1) then
      i0 = 0
    else
      i0 = 1
    end if

    do k = 1,nsrc

      id_src = id_src+1

      do i = i0,num_instr0

       if (instflg(i)) then

        if (i == 0) then

! Write instrument summary
          call mlout(1,ll,id_src,insrc(k),i,0,id_cluster, &
               sctst(k),esctst(k),xms(k),yms(k),exms(k),eyms(k), &
               ext(k),ext_err(k),exi_mlt(k),ext_ml(k), &
               bkgt(k),dum,flxt(k),eflxt(k),ratet(k),eratet(k), &
               ra(k),dec(k),eradec(k),lii(k),bii(k),0.,0.,0.,0.,0.,0., &
               cutrad,hew,flags)

        else
! Energy band summary only required if more than one band
          if (num_image0(i) > 1) then
            j0 = 0
          else
            j0 = 1
          end if

          do j = j0,num_image0(i)

! Set energy band id to 0
            if (j == 0) then
              jj = 0
            else
              jj = j
            end if 
! Set energy band to 9 for XID band
            if (jj == xid_arr0(i)) jj = 9

! Write energy band summary
            if (j == 0) then
              call mlout(2,ll,id_src,insrc(k),i,jj,id_cluster, &
                 sctsi(i,k),esctsi(i,k),xms(k),yms(k),exms(k),eyms(k), &
                 ext(k),ext_err(k),exi_mli(i,k),ext_ml(k), &
                 bkgi(i,k),dum,flxi(i,k), &
                 eflxi(i,k),ratei(i,k),eratei(i,k),ra(k),dec(k),eradec(k), &
                 lii(k),bii(k),hr1(i,k),hr1_err(i,k),hr2(i,k),hr2_err(i,k), &
                 hr3(i,k),hr3_err(i,k),cutrad,hew,flags)
            end if

! Write results per band and per instrument

            if (j /= 0) then
              ipoint=ip(i,j)
              call mlout(3,ll,id_src,insrc(k),i,jj,id_cluster, &
                   scts0(i,j,k), escts0(i,j,k),xms(k),yms(k), &
                   exms(k),eyms(k),ext(k),ext_err(k), &
                   exi_ml0(i,j,k),ext_ml(k),bkg0(i,j,k), &
                   expos0(i,j,k),flx0(i,j,k),eflx0(i,j,k), &
                   rate0(i,j,k),erate0(i,j,k),ra(k),dec(k),eradec(k), &
                   lii(k),bii(k),0.,0.,0.,0.,0.,0.,cutrad,hew,flags)
              end if 

          end do
         end if      
        end if
      end do
    end do

  end subroutine wr_stab

  !---------------------------------------------------------------------
  subroutine mlout (k,ll,id_src_d2,id_src_d1,id_inst,id_band,id_cluster,&
                    scts,scts_err,x_ima,y_ima,x_ima_err,y_ima_err,    &
                    ext,ext_err,exi_ml,ext_ml,bg_map, &
                    exp_map,flux,flux_err,rate,rate_err,ra,dec,       &
                    radec_err,lii,bii,hr1,hr1_err,hr2,hr2_err,hr3,hr3_err, &
                    cutrad,hew,flags)
  !---------------------------------------------------------------------
        
    integer, intent(IN) :: k,id_src_d2,id_src_d1,id_inst, &
                           id_band,id_cluster
    real(kind=single), intent(IN) :: scts,scts_err,x_ima,y_ima,        &
                        x_ima_err,y_ima_err,ext,ext_err,&
                        exi_ml,ext_ml, &
                        bg_map,exp_map,flux,flux_err,rate,rate_err,    &
                        radec_err,hr1,hr1_err,hr2,hr2_err,hr3,hr3_err, &
                        cutrad,hew
    real(kind=double), intent(IN) :: ra,dec,lii,bii
    character(len=*), intent(IN) :: flags
    integer, intent(INOUT) :: ll

    integer, parameter :: ncol = 35
    integer :: i, l1
    character(len=1024) :: image

    type(DataSetT), save :: Set
    type(DataSetT) :: set0, set1
    type(TableT), save :: tab
    type(ColumnT), dimension(40), save :: c 
    type(ColumnT), save :: sp36, sp37, sp38

    integer, dimension(0:max_sou), save :: ia0, ia1, ia2, ia3, ia4, ia5 
    real(kind=single), dimension(0:max_sou), save :: ra6, ra7, ra8, &
      ra9, ra10, ra11, ra12, ra13, ra14, ra15, ra16, ra17, ra18, ra19, &
      ra20, ra21, ra24, ra25, ra26, ra27, ra28, ra29, ra30, ra31, ra32
    real(kind=double), dimension(0:max_sou), save :: da22, da23, da34, da35

    integer, dimension(:), pointer :: ip0, ip1, ip2, ip3, ip4, ip5 
    real(kind=single), dimension(:), pointer :: rp6, rp7, rp8, &
      rp9, rp10, rp11, rp12, rp13, rp14, rp15, rp16, rp17, rp18, rp19, &
      rp20, rp21, rp24, rp25, rp26, rp27, rp28, rp29, rp30, rp31, rp32
    real(kind=double), dimension(:), pointer :: dp22, dp23, dp34, dp35

    character(len=12) :: valflag = "            ", &
                         verflag = "            "
    character(len=40) :: vcmflag = "                                        "
    integer :: ii

    if (k == 0) then
 
      ! ll=-ll
      l1=0     
      set = dataSet (outtab, CREATE)
      if (phofla) then
        image = c_evlist(1)   
      else
        image = c_image(1) 
      end if
      set0 = dataSet (image, READ) 
      call cp_head (set0, set, "EPIC EXPOSURE  ML SOURCE LIST", xid_arr, 4)
      call release(set0)
       
      set1 = DataSet(c_d1list, read)
      if (hasAttribute(set1,"BOXMODE")) &
        call setAttribute(set, "BOXMODE", &
                          stringAttribute(set1, "BOXMODE"), &
                          "Mode of eboxdetect input source list" ) 
      call release(set1)  
      call setAttribute(set, "HDUCLASS", "OGIP    ", &
                              "Format conforms to OGIP/GSFC conventions")
      call setAttribute(set, "HDUCLAS1", "SRCLIST   ", &
                              "File contains a source list")

      tab = addTable (set, "SRCLIST", ll)

!      l1 = l1+1
!      c(l1) = addColumn (tab, "SRC_NUM",  Integer32, " ")
!      ip0 => int32Data(c(l1))
!      ip0 = ia0(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "ML_ID_SRC",  Integer32, " ")
      ip1 => int32Data(c(l1))
      ip1 = ia1(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "BOX_ID_SRC",  Integer32, " ")
      ip2 => int32Data(c(l1))
      ip2 = ia2(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "ID_INST",    Integer32, " ")
      ip3 => int32Data(c(l1))
      ip3 = ia3(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "ID_BAND",    Integer32, " ")
      ip4 => int32Data(c(l1))
      ip4 = ia4(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "ID_CLUSTER", Integer32, " ")
      ip5 => int32Data(c(l1))
      ip5 = ia5(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "SCTS",       Real32, "counts")
      rp6 => real32Data(c(l1))
      rp6 = ra6(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "SCTS_ERR",   Real32, "counts")
      rp7 => real32Data(c(l1))
      rp7 = ra7(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "X_IMA",      Real32, "image pixels")
      rp8 => real32Data(c(l1))
      rp8 = ra8(0:ll-1) 

      l1 = l1+1
      c(l1) = addColumn (tab, "X_IMA_ERR",  Real32, "image pixels")
      rp9 => real32Data(c(l1))
      rp9 = ra9(0:ll-1)
   
      l1 = l1+1
      c(l1) = addColumn (tab, "Y_IMA",      Real32, "image pixels")
      rp10 => real32Data(c(l1))
      rp10 = ra10(0:ll-1) 

      l1 = l1+1
      c(l1) = addColumn (tab, "Y_IMA_ERR",  Real32, "image pixels")
      rp11 => real32Data(c(l1))
      rp11 = ra11(0:ll-1) 

      l1 = l1+1
      c(l1) = addColumn (tab, "EXT",        Real32, "image pixels")
      rp12 => real32Data(c(l1))
      rp12 = ra12(0:ll-1)
      do i=0,ll-1        
        if (.not.extfla .or. ra15(i)<dmlext) call settoNull(c(l1),i)
      end do

      l1 = l1+1
      c(l1) = addColumn (tab, "EXT_ERR",    Real32, "image pixels")
      rp13 => real32Data(c(l1))
      rp13 = ra13(0:ll-1)
      do i=0,ll-1        
        if (.not.extfla .or. ra15(i)<dmlext) call settoNull(c(l1),i)
      end do

      l1 = l1+1
      c(l1) = addColumn (tab, "DET_ML",     Real32, " ")
      rp14 => real32Data(c(l1))
      rp14 = ra14(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "EXT_ML",     Real32, " ")
      rp15 => real32Data(c(l1))
      rp15 = ra15(0:ll-1)
      do i=0,ll-1        
        if (.not.extfla .or. ra15(i)<dmlext) call settoNull(c(l1),i)
      end do

      l1 = l1+1
      c(l1) = addColumn (tab, "BG_MAP",     Real32, "counts/pixel")
      rp16 => real32Data(c(l1))
      rp16 = ra16(0:ll-1)      

      l1 = l1+1
      c(l1) = addColumn (tab, "EXP_MAP",    Real32, "seconds")
      rp17 => real32Data(c(l1))
      rp17 = ra17(0:ll-1)
      do i=0,ll-1        
        if (ia4(i) == 0) call settoNull(c(l1),i)
      end do

      l1 = l1+1
      c(l1) = addColumn (tab, "FLUX",       Real32, "cgs")
      rp18 => real32Data(c(l1))
      rp18 = ra18(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "FLUX_ERR",   Real32, "cgs")
      rp19 => real32Data(c(l1))
      rp19 = ra19(0:ll-1) 

      l1 = l1+1
      c(l1) = addColumn (tab, "RATE",       Real32, "counts/s")
      rp20 => real32Data(c(l1))
      rp20 = ra20(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "RATE_ERR",   Real32, "counts/s")
      rp21 => real32Data(c(l1))
      rp21 = ra21(0:ll-1) 

      l1 = l1+1
      c(l1) = addColumn (tab, "RA",         Real64, "deg")
      dp22 => real64Data(c(l1))
      dp22 = da22(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "DEC",        Real64, "deg")
      dp23 => real64Data(c(l1))
      dp23 = da23(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "RADEC_ERR",  Real32, "arcsec")
      rp24 => real32Data(c(l1))
      rp24 = ra24(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "LII",        Real64, "deg")
      dp34 => real64Data(c(l1))
      dp34 = da34(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "BII",        Real64, "deg")
      dp35 => real64Data(c(l1))
      dp35 = da35(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "HR1",        Real32, " ")
      rp25 => real32Data(c(l1))
      rp25 = ra25(0:ll-1)
      do i=0,ll-1        
        if (ia3(i)==0.or.ia4(i)/=0.or.ra25(i)<-10.0) call settoNull(c(l1),i)
      end do

      l1 = l1+1
      c(l1) = addColumn (tab, "HR1_ERR",    Real32, " ")
      rp26 => real32Data(c(l1))
      rp26 = ra26(0:ll-1)
      do i=0,ll-1        
        if (ia3(i)==0.or.ia4(i)/=0.or.ra26(i)<-10.0) call settoNull(c(l1),i)
      end do

      l1 = l1+1
      c(l1) = addColumn (tab, "HR2",        Real32, " ")
      rp27 => real32Data(c(l1))
      rp27 = ra27(0:ll-1)
      do i=0,ll-1        
        if (ia3(i)==0.or.ia4(i)/=0.or.ra27(i)<-10.0) call settoNull(c(l1),i)
      end do

      l1 = l1+1
      c(l1) = addColumn (tab, "HR2_ERR",    Real32, " ")
      rp28 => real32Data(c(l1))
      rp28 = ra28(0:ll-1)
      do i=0,ll-1        
        if (ia3(i)==0.or.ia4(i)/=0.or.ra28(i)<-10.0) call settoNull(c(l1),i)
      end do

      l1 = l1+1
      c(l1) = addColumn (tab, "HR3",        Real32, " ")

      rp29 => real32Data(c(l1))
      rp29 = ra29(0:ll-1)
      do i=0,ll-1        
       if (ia3(i)==0.or.ia4(i)/=0.or.ra29(i)<-10.0) call settoNull(c(l1),i)
      end do

      l1 = l1+1
      c(l1) = addColumn (tab, "HR3_ERR",    Real32, " ")
      rp30 => real32Data(c(l1))
      rp30 = ra30(0:ll-1)
      do i=0,ll-1        
        if (ia3(i)==0.or.ia4(i)/=0.or.ra30(i)<-10.0) call settoNull(c(l1),i)
      end do

      l1 = l1+1
      c(l1) = addColumn (tab, "CUTRAD",     Real32, "pixels") ! unit ???
      rp31 => real32Data(c(l1))
      rp31 = ra31(0:ll-1)

      call neighbour(ra8,ra10,ra32,ll)

      l1 = l1+1
      c(l1) = addColumn (tab, "DIST_NN",    Real32, "arcsec") ! unit ???
      rp32 => real32Data(c(l1))
      rp32 = ra32(0:ll-1)

      l1 = l1+1
      c(l1) = addColumn (tab, "VAL_FLAG",   String, " ", (/12/) )
      sp36=c(l1)

      l1 = l1+1
      c(l1) = addColumn (tab, "VER_FLAG",   String, " ", (/12/) )
      sp37=c(l1)

      l1 = l1+1
      c(l1) = addColumn (tab, "VER_COMM",   String, " " , (/40/) )
      sp38=c(l1)

      do ii=0,ll-1
        call setStringCell(sp36,ii,valflag) 
        call setStringCell(sp37,ii,verflag) 
        call setStringCell(sp38,ii,vcmflag) 
      end do
          
      do i=1,l1
        call release(c(l1))
      end do
      call release (tab)
      call release (set)

    else if (ll < max_sou) then   

      ia1(ll) = id_src_d2
      ia2(ll) = id_src_d1
      ia3(ll) = id_inst
      ia4(ll) = id_band
      ia5(ll) = id_cluster
      ra6(ll) = scts
      ra7(ll) = scts_err
      ra8(ll) = x_ima
      ra9(ll) = x_ima_err
      ra10(ll) = y_ima 
      ra11(ll) = y_ima_err
      ra12(ll) = ext
      ra13(ll) = ext_err
      ra14(ll) = exi_ml
      ra15(ll) = ext_ml
      ra16(ll) = bg_map
      if (k == 3) then
        ra17(ll) = exp_map
      end if
      ra18(ll) = flux
      ra19(ll) = flux_err
      ra20(ll) = rate
      ra21(ll) = rate_err
      da22(ll) = ra
      da23(ll) = dec
      ra24(ll) = radec_err
      ra25(ll) = hr1
      ra26(ll) = hr1_err
      ra27(ll) = hr2
      ra28(ll) = hr2_err 
      ra29(ll) = hr3
      ra30(ll) = hr3_err 
      ra31(ll) = cutrad
      ra32(ll) = 0.
      da34(ll) = lii
      da35(ll) = bii
      ll = ll+1 

    else if (ll == max_sou) then
      call warning("BufferOverflow  ","More than 10000 sources detected") 
    end if

  end subroutine mlout

  subroutine neighbour (xms,yms,dist,nsrc)

  real(kind=single), dimension(0:max_sou), intent(in) :: xms, yms
  integer, intent(in) :: nsrc 
  real(kind=single), dimension(0:max_sou), intent(out) :: dist
  real(kind=single) d2,d2save
  integer :: i,j
  
  do i=0,nsrc-1
    d2save=1e10
    do j=0,nsrc-1
      d2=(xms(i)-xms(j))**2+(yms(i)-yms(j))**2
      if (d2 > 0.1 .and. d2 < d2save) then
        d2save=d2
      end if  
    end do
    dist(i)=sqrt(d2save)*pixim
  end do  

  end subroutine neighbour

  subroutine wr_srcmap

      real(kind=single), dimension(:,:), pointer :: r4arr

      integer :: ix, iy, ipoint 

      character(len=64) :: output
      
      type(DataSetT) :: set, set0
      Type(ArrayT) :: arr

      do ipoint=1,nim

        write(output,'(a,i1,a)')'srcmap',ipoint,'.fits'
        set = dataset(output, write)
        arr = addArray(set, "PRIMARY", Real32, npix)
        r4arr => real32array2data(arr)
        do ix=1,npix(1)
          do iy=1,npix(2)
            r4arr(ix-1,iy-1)=bg_image(ipoint,ix,iy)
          end do
        end do  
!      set0 = dataset(refnam, read)
!      call cp_head (set0, set, ident, 2)
!      call release(set0)
       call release(arr)
       call release(set)   

      end do
 
      end subroutine wr_srcmap

end module eml_io













