module emlfit

  use param
  use dal
  use sla
  use edetect_io
  use emldata
  use psf 


  implicit none

  private

  public mulsou, model, mlfit, funct, like, ordr, pordr

contains

  !---------------------------------------------------------------------
      subroutine mulsou (nsrc,xms,yms,sigma,scts,ra,dec,lii,bii, &
                         exms,eyms,esigma,escts,eradec,exi_ml,ext_ml,ecut2)
  !---------------------------------------------------------------------

      real, intent(in) :: ecut2
      integer, intent(inout) :: nsrc
      real, dimension(max_fit), intent(inout) :: xms, yms, sigma, &
                                                 exms, eyms, esigma, ext_ml
      real(kind=double), dimension(max_fit), intent(out) :: ra, dec, lii, bii
      real(kind=single), dimension(max_fit) ::  eradec
      real, dimension(max_ima,max_fit), intent(inout) :: scts, escts, exi_ml 

      real ::       yfit1(max_events),yfit2(max_ima),a(max_par), &
                    deltaa(max_par),sigmaa(max_par),afit(max_par), &
                    delfit(max_par),sigfit(max_par),yfit3,dml(max_fit), &
                    fe(max_par),amerk(max_par),like0,lik_arr(max_ima), &
                    likmerk,lik_amerk(max_ima),likold,df,nterml, &
                    felhi,fello, flamda, sc, ecut !, buff(640,640)
      real(kind=double) :: xr,yr,dra,ddec,r0r,d0r,rr,dr,sin_pa,cos_pa, &
                    lr,br
      integer ::    ierr,ioff0,ioff1,ioff,i,ifel,nspar, &
                    npsou,ipoint,iordr(max_par),iorde(max_par), &
                    j,insrc(max_fit), &
                    ix,iy,l,m,n,nlauf,ntermf,ntfix,nffix,np,nima
      logical       done

      real, parameter :: delt = 1e-4
    
      ierr=0
      ntfix = 0
      nlauf=0

      nspar=3
      npsou=nim+nspar
      np=npsou*nsrc
      do i=1,nsrc
        ioff=npsou*(i-1)
        sc=0.0
        do j=1,nim
          a(ioff+3+j)=max(scts(j,i),0.0)
          deltaa(ioff+3+j)=max(1.0*sqrt(max(scts(j,i),0.0)),1.0)
          sc=sc+a(ioff+3+j)
        end do
        a(ioff+1)=xms(i)
        deltaa(ioff+1)=2.0/sqrt(sc)
        a(ioff+2)=yms(i)
        deltaa(ioff+2)=2.0/sqrt(sc)
        a(ioff+3)=0.    
        deltaa(ioff+3)=1.0       ! ??
      end do

    done = .false.
    do while (.not. done)
      iordr(1)=0
      call ordr(np,np,iordr)                                           

      ntermf=np
      !print *,"NTERMF-:",ntermf
! optionally fix sigmas in fit
      if (.not. extfla) then
        do i=1,nsrc
          ioff=npsou*(i-1)
          call ordr(np,ioff+3,iordr)
        end do
        ntermf=ntermf-nsrc
        ntfix=nsrc
        nspar=nspar-1
      end if
      !print *,"NTERMF0:",ntermf

! optionally fix positions in fit
      if (.not. posfla) then
        do n=1,nsrc
          call ordr(ntermf,3*(n-1)+1,iordr)
          call ordr(ntermf,3*(n-1)+2,iordr)
        enddo
        ntermf=ntermf-2*nsrc
        ntfix=ntfix+2*nsrc
        nspar=nspar-2
      endif

      do nlauf=1,15
        flamda=1e-2                                                    
        call pordr(np,a,afit,iordr,1)
        call pordr(np,deltaa,delfit,iordr,1)
        call pordr(np,sigmaa,sigfit,iordr,1)                            
        call mlfit(ntermf,afit,delfit,sigfit,iordr,xms,yms,ecut2, &
                                flamda,yfit1,like0,lik_arr,nsrc,ntfix,ierr)
        call pordr(np,a,afit,iordr,-1)                                   
        call pordr(np,deltaa,delfit,iordr,-1)                            
        call pordr(np,sigmaa,sigfit,iordr,-1)                            
        if(abs(likold-like0).le.delt) exit                   
        likold=like0                                                    
      end do

      df=npts-np                                                       

! save parameters ......
      likmerk=like0
      lik_amerk=lik_arr
      do n=1,np
        amerk(n)=a(n)
      enddo

! calculate extent likelihood
      if (extfla) then
        do n=nsrc,1,-1
          ioff=npsou*(n-1)
          if (nsrc == 1) then
            a(3)=0.
            call pordr(np,a,afit,iordr,1)
            call funct(afit,iordr,yfit1,yfit2,yfit3,nsrc,np,xms,yms,ecut2)
            call pordr(np,a,afit,iordr,-1)
            call like(like0,lik_arr,yfit1,yfit2)            
          else
! zero extent of relevant component
            a(ioff+3)=0. 
            iorde=iordr
            call ordr(ntermf,ioff+3,iorde)
            nffix=1

            do nlauf=1,15
              flamda=1e-2
              call pordr(np,a,afit,iorde,1)
              call pordr(np,deltaa,delfit,iorde,1)
              call pordr(np,sigmaa,sigfit,iorde,1)
              call mlfit(ntermf-nffix,afit,delfit,sigfit,iordr,xms,yms, &
                   ecut2,flamda,yfit1,like0,lik_arr,nsrc,ntfix+nffix,ierr)
              call pordr(np,a,afit,iorde,-1)
              call pordr(np,deltaa,delfit,iorde,-1)
              call pordr(np,sigmaa,sigfit,iorde,-1)
              if(abs(likold-like0).le.delt) exit
              likold=like0                        
            end do                            
          end if
          ext_ml(n)=like0-likmerk
! store point source fit if extent is not significant
          if (ext_ml(n) < dmlext) then
            do j=1,nim+3
              amerk(ioff+j)=a(ioff+j)
            end do
          end if  
        end do
      end if

! now calculate the likelihood of each of the sources
      do n=nsrc,1,-1
        ioff=npsou*(n-1)

        if(nsrc.eq.1)then

! simply calculate likelihood for one source only
          do j=1,nim
            a(3+j)=0.
          end do
          call pordr(np,a,afit,iordr,1)
          call funct(afit,iordr,yfit1,yfit2,yfit3,nsrc,np,xms,yms,ecut2)
          call pordr(np,a,afit,iordr,-1)
          call like(like0,lik_arr,yfit1,yfit2)

        else

! zero flux of relevant component
          nffix=0
          if (posfla) then
            call ordr(ntermf,ioff+1,iordr)
            call ordr(ntermf,ioff+2,iordr)
            nffix=nffix+2
          end if
          if (extfla .and. ext_ml(n) >= dmlext) then
            call ordr(ntermf,ioff+3,iordr)
            nffix=nffix+1
          end if
          do j=1,nim
            a(ioff+3+j)=0.
            call ordr(ntermf,ioff+3+j,iordr)
            nffix=nffix+1
          end do
      
! fix parameters 
          do nlauf=1,15
            flamda=1e-2
            call pordr(np,a,afit,iordr,1)
            call pordr(np,deltaa,delfit,iordr,1)
            call pordr(np,sigmaa,sigfit,iordr,1)
            call mlfit(ntermf-nffix,afit,delfit,sigfit,iordr,xms,yms, &
                 ecut2,flamda,yfit1,like0,lik_arr,nsrc,ntfix+nffix,ierr)
            call pordr(np,a,afit,iordr,-1)
            call pordr(np,deltaa,delfit,iordr,-1)
            call pordr(np,sigmaa,sigfit,iordr,-1)
            if(abs(likold-like0).le.delt) exit
            likold=like0                        
          end do                            
        endif

        !dml(n)=like0-likmerk
        do j=1,nim 
          exi_ml(j,n)=lik_arr(j)-lik_amerk(j)
        end do
        dml(n)=0. 
        do j=1,num_instr
          nima=num_image(j)
          if (xid_arr(j)>0) nima=xid_arr(j)-1
          do i=1,nima
            ipoint=ip(j,i)
            dml(n)=dml(n)+exi_ml(ipoint,n)
          end do
        end do
        do j=1,nim 
          exi_ml(j,n)=max(lik_arr(j)-lik_amerk(j),0.)
        end do

        if(dml(n).lt.dmlmin)then

! throw out component
          do l=n+1,nsrc
            ioff0=npsou*(l-1)
            ioff1=npsou*l
            do j=1,3
              a(ioff0+j)=a(ioff1+j)
            end do          
            do j=1,nim
              a(ioff0+3+j)=a(ioff1+3+j)
            enddo
            insrc(l-1)=insrc(l)
          enddo
          nsrc=nsrc-1
          if(nsrc.eq.0)then
!            write(6,*)' no source accepted'
            return
          endif
          np=nsrc*npsou
          exit
        else  

! get back best-fit parameters
          do m=1,np
            a(m)=amerk(m)
          enddo
          like0=likmerk
          done = .true.
        endif
      enddo
     end do 

! call function to reset the model yfit
      call pordr(np,a,afit,iordr,1)                               
      call funct(afit,iordr,yfit1,yfit2,yfit3,nsrc,np,xms,yms,ecut2)
      call pordr(np,a,afit,iordr,-1)                               

! error loop begins   
      if (errfla) then 
      !print *,"ERROR"

! now position errors for position fixing
!       npsou=nim+nspar
        do i=1,nsrc
          do j=1,npsou
            if (posfla .and. j <= 2 .or. &
                extfla .and. j == 3 .or. &
                j >= 4) then

              ifel=npsou*(i-1)+j

              call ordr(ntermf,ifel,iordr)

              flamda=1.0e-2
              call pordr(np,a,afit,iordr,1)
              call pordr(np,deltaa,delfit,iordr,1)
              call pordr(np,sigmaa,sigfit,iordr,1)
              call felsu(ntermf,afit,delfit,sigfit,iordr,xms,yms, &
                         flamda,nsrc,like0,fello,felhi,ntfix,ecut2)

              call pordr(np,a,afit,iordr,-1)
              call pordr(np,deltaa,delfit,iordr,-1)
              call pordr(np,sigmaa,sigfit,iordr,-1)
             
              fe(ifel)=0.5*(max(fello,0.)+max(felhi,0.))

            end if
          end do
        end do
      
      else
        fe = 0.0
      end if
                                                                      
! assign output values
      do i=1,nsrc
        ioff=npsou*(i-1)
        xms(i)=a(ioff+1)
        exms(i)=fe(ioff+1)
        yms(i)=a(ioff+2)
        eyms(i)=fe(ioff+2)
        sigma(i)=a(ioff+3)
        if (sigma(i) .gt. 0.) then
          esigma(i)=fe(ioff+3)
        else
          esigma(i)=0.
        end if 
        do j=1,nim
          scts(j,i)=a(ioff+3+j)
          escts(j,i)=fe(ioff+3+j)
        end do

        cos_pa = dcos(crota2 * pi/180d0)
        sin_pa = dsin(crota2 * pi/180d0)
        xr = (xms(i)-crpix1) * crdelt1 * pi/180d0
        yr = (yms(i)-crpix2) * crdelt2 * pi/180d0
        dra =  xr * cos_pa - yr * sin_pa
        ddec = xr * sin_pa + yr * cos_pa
        r0r = crval1 * pi/180d0
        d0r = crval2 * pi/180d0
        call antisp(dra,ddec,r0r,d0r,rr,dr)
        ra(i) = rr * 180d0/pi
        dec(i) = dr *180d0/pi
        eradec(i)  = sqrt(exms(i)**2+eyms(i)**2) * pixim

        call sla_EQGAL(rr,dr,lr,br)
        lii(i) = lr * 180d0/pi
        bii(i) = br * 180d0/pi

      enddo

! updatebackground map
      ecut = sqrt(ecut2)
      call model(yfit1,yfit2,yfit3,xms,yms,sigma,scts,nsrc,ecut)

      l=1
      do i=1,num_instr
        do j=1,num_image(i)
          ipoint=ip(i,j)       
          do iy=mask%iy_min,mask%iy_max
            do ix=mask%ix_min,mask%ix_max
              if (mask%mask(ix-mask%ix_min+1,iy-mask%iy_min+1).eq.1) then 

                !bg_image(ipoint,ix,iy) = bg_image(ipoint,ix,iy)+yfit1(l)
                bg_image(ipoint,ix,iy) = yfit1(l)
                l=l+1

              end if            
            end do
          end do 
        end do
      end do
                                                        
      end subroutine mulsou   

  !----------------------------------------------------------------------
  subroutine ordr(nterms,nfix,iordr)
  !----------------------------------------------------------------------

    integer     i,iordr(max_par),io(max_par),nfix,nnfix,nterms

    if(iordr(1).eq.0)then
      do i=1,max_par
        iordr(i)=i
      end do
    endif

    do i=1,max_par
      if(iordr(i).eq.nfix) exit
      io(i)=iordr(i)
    end do

    nnfix=i

    do i=nnfix,nterms-1
      io(i)=iordr(i+1)
    end do

    io(nterms)=nfix

    if(nterms.ne.max_par) then
      do i=nterms+1,max_par
        io(i)=iordr(i)
      end do
    end if 

    iordr(1:max_par)=io(1:max_par)

  end subroutine ordr

  !----------------------------------------------------------------------
  subroutine pordr(nterms,a,b,iordr,mode)
  !----------------------------------------------------------------------

    real          a(max_par),b(max_par)
    integer       i,mode,nterms,iordr(max_par)

    if(mode.eq.1) then
      do i=1,nterms
        b(i)=a(iordr(i))
      enddo
    else

! andere richtung
      do i=1,nterms
        a(iordr(i))=b(i)
      enddo
    end if

  end subroutine pordr

  !----------------------------------------------------------------------
  function cage(array,n,vallo,valhi)
  !----------------------------------------------------------------------

    real ::        array(max_par),cage,value,vallo,valhi
    integer ::     ierr,n 

    ierr=0
!
! used to cage fit parameters to reasonable values
    value=array(n)

    if (value.lt.vallo) then
      array(n)=vallo
!      write(6,*)' allowed values ',vallo,valhi
    else if (value.gt.valhi) then
      array(n)=valhi
!      write(6,*)' allowed values ',vallo,valhi
    endif
    cage=array(n)

  end function cage    
      
  !----------------------------------------------------------------------
  subroutine cagexy (x1,y1,x0,y0,rcut2)
  !----------------------------------------------------------------------

    real     x0,y0,x1,y1,rcut2  
    real     ds
    integer  ndat

    ds=sqrt(((x1-x0)**2+(y1-y0)**2)/rcut2)
    if (ds.gt.1.) then
      x1 = x0+(x1-x0)/ds
      y1 = y0+(y1-y0)/ds
    end if
 
  end subroutine cagexy

  !----------------------------------------------------------------------
  subroutine like(lik,lik_arr,yfit1,yfit2)
  !----------------------------------------------------------------------

    real, dimension(max_events), intent(in) :: yfit1
    real, dimension(max_ima), intent(in) :: yfit2

    real, dimension(max_ima), intent(out) :: lik_arr
    real, intent(out) :: lik

    real :: s1, s2
    integer :: k,ipoint 

! accumulate statistics
    lik_arr(1:nim)=0.
    do k=1,npts
      ipoint=event(k)%ipt
      lik_arr(ipoint)=lik_arr(ipoint)+event(k)%n*log(yfit1(k))
    end do
   
    lik_arr(1:nim)=lik_arr(1:nim)-yfit2(1:nim)
    lik_arr(1:nim)=-2.*lik_arr(1:nim)

    lik=0. 
    do ipoint=1,nim
      lik=lik+lik_arr(ipoint)
    end do 

!    s1=0.
!    s2=0.
!    do k=1,npts
!      s1=s1+yfit1(k)
!      s2=s2+event(k)%n
!    end do

   end subroutine like

  !----------------------------------------------------------------------
  subroutine mlfit(nterms,a,deltaa,sigmaa,iordr,xms,yms,ecut2,flamda, &
                   yfit1,like0,lik_arr,is,nfel,ierr)
  !----------------------------------------------------------------------

    integer :: i,j,k, ntest1, ntest2
    integer :: ntermf,nterms,nfree,nfel,is,ierr,iprint,iordr(max_par)
    real(kind=double) :: huge,det
    real :: a(max_par),deltaa(max_par),sigmaa(max_par)
    real :: xms(max_fit),yms(max_fit),b(max_par)
    real :: lik_arr(max_ima),yfit1(max_events),ytemp1(max_events)
    real :: yfit2(max_ima),yfit3,ytemp3
    !! real :: derv1(max_par,max_events),derv3(max_par)
    real :: derv3(max_par)
    real, allocatable :: derv1(:,:)
    real :: like0,like1,aj,delta,delta2,ecut2,flamda,test1,test2 
    real(kind=double) ::array(max_par,max_par),alpha(max_par,max_par), &
                        beta(max_par)      

    real, parameter :: small = 1e-6
!   small=tiny(small)  

    !allocate(derv1(max_par,max_events))
    allocate(derv1(nterms,npts))

    ntermf=nterms+nfel
    nfree=npts-nterms
    if (nfree.le.0) &
      call error("FitFailed","degrees of freedom less or equal 0")

    do j=1,nterms
      aj=a(j)

      delta=deltaa(j)
      delta2=2.0*delta

      a(j)=aj+delta
      call funct(a,iordr,yfit1,yfit2,yfit3,is,nterms+nfel,xms,yms,ecut2)

      a(j)=aj-delta
      call funct(a,iordr,ytemp1,yfit2,ytemp3,is,nterms+nfel,xms,yms,ecut2)

      derv1(j,1:npts)=(yfit1(1:npts)-ytemp1(1:npts))/delta2
      derv3(j)=(yfit3-ytemp3)/delta2

      a(j)=aj
    end do
        
    beta(1:nterms)=0.
    alpha(1:nterms,1:nterms)=0.          

    call funct(a,iordr,yfit1,yfit2,yfit3,is,nterms+nfel,xms,yms,ecut2)

!      do i=1,npts
!       do j=1,nterms 
!          beta(j)=beta(j)+(event(i)%n/yfit1(i)*derv1(j,i)  &
!                                                     -derv3(j))
!          do k=1,j 
!            alpha(j,k)=alpha(j,k)+ &
!                   event(i)%n/yfit1(i)**2*derv1(j,i)*derv1(k,i)
!          end do
!        end do
!      end do

     do i=1,npts
       do j=1,nterms
!        beta(j)=beta(j)+(event(i)%n-yfit1(i))/max(event(i)%n,1)*derv1(j,i) 
        beta(j)=beta(j)+(event(i)%n-yfit1(i))/yfit1(i)*derv1(j,i) 
        do k=1,j 
!         alpha(j,k)=alpha(j,k)+derv1(j,i)*derv1(k,i)/max(event(i)%n,1)
          alpha(j,k)=alpha(j,k)+derv1(j,i)*derv1(k,i)/yfit1(i)
        end do
      end do
    end do

    do j=1,nterms
      do k=1,j
        alpha(k,j)=alpha(j,k)
      end do
    end do

    call funct(a,iordr,yfit1,yfit2,yfit3,is,nterms+nfel,xms,yms,ecut2)
    call like(like1,lik_arr,yfit1,yfit2)

    do
      
      do j=1,nterms
!       print *, "alpha",alpha(j,j)
        do k=1,nterms
          if(j.eq.k)then
            array(j,k)=1.0d0
          else 
            array(j,k)=alpha(j,k)/dsqrt(dabs(alpha(j,j))+small) &
                                 /dsqrt(dabs(alpha(k,k))+small)
          endif
        end do
        array(j,j)=1.0d0+flamda
      end do
 
      call mati(array,nterms,det)

!      b(1:nterms)=a(1:nterms)
      b = a

      do j=1,nterms
        do k=1,nterms
          b(j)=b(j)+beta(k)*array(j,k)/dsqrt(dabs(alpha(j,j))+small) &
                                      /dsqrt(dabs(alpha(k,k))+small)
        end do 
      end do

      call funct(b,iordr,yfit1,yfit2,yfit3,is,nterms+nfel,xms,yms,ecut2)
      call like(like0,lik_arr,yfit1,yfit2)

      if (like0-like1.lt.0.) then
        exit
      else
        flamda=10.*flamda
        if (flamda.ge.1e10) exit
      end if

    end do

    a(1:nterms)=b(1:nterms)
    do j=1,nterms
      sigmaa(j)=dsqrt(dabs(array(j,j))/(dabs(alpha(j,j))+small))
    end do
    flamda=flamda/10.

    deallocate (derv1)

  end subroutine mlfit

  !----------------------------------------------------------------------
  subroutine funct(afit,iordr,yfit1,yfit2,yfit3,is,np,xms,yms,ecut2)
  !------------------------------------------------------------------------
      
    real :: yfit1(max_events),yfit2(max_ima),yfit3,a(max_par),afit(max_par), & 
            x0(max_fit),y0(max_fit),sig(max_fit),scts(max_ima,max_fit), &
            xms(max_fit),yms(max_fit),ecut2,ecut
    integer :: i,n,ierr,iordr(max_par),ipoint,iprint,is,np, &
               ndat,npars

    ierr = 0

    call pordr(np,a,afit,iordr,-1)                               

! cage parameters
    npars=nim+3
    do n=1,is 
      ndat=(n-1)*npars+1
      call cagexy (a(ndat),a(ndat+1),xms(n),yms(n),ecut2)
      x0(n)=a(ndat)
      y0(n)=a(ndat+1)
      ndat=(n-1)*npars+3
      sig(n)=cage(a,ndat,0.,200.)
      do i=1,nim
        ndat=(n-1)*npars+i+3
        scts(i,n)=cage(a,ndat,0.,1e10)
      end do
    end do 

! calculate expectation value of model + background
    ecut = sqrt(ecut2)
    call model(yfit1,yfit2,yfit3,x0,y0,sig,scts,is,ecut)

    call pordr(np,a,afit,iordr,1)                                

  end subroutine funct
           
  !----------------------------------------------------------------------
  subroutine model(yfit1,yfit2,yfit3,x0,y0,sig,scts,is,ecut)
  !----------------------------------------------------------------------
 
    real, dimension(max_events), intent(out) :: yfit1
    real, dimension(max_ima), intent(out) :: yfit2
    real, intent(out) :: yfit3

    real, dimension(max_fit), intent(in) :: x0, y0, sig
    real, dimension(max_ima,max_fit), intent(in) :: scts

    real, intent(in) :: ecut
    integer, intent(in) :: is
    

    real, dimension(max_ima) :: offax, posang

    real :: ene,x1,y1,xd,yd
    real :: s1,s2,s3,fudge
    integer :: i,j,l,n,ix,iy,ix0,ix1,iy0,iy1,ipoint

    l=1
    yfit3=0.
    fudge=1.0
    do i=1,num_instr

      if (num_evlist(i).eq.0) then

        s3=0.
        do j=1,num_image(i)
          ipoint=ip(i,j)
          ene=energy(ipoint)

          s2=0.
          ix0 = mask%ix_min
          ix1 = mask%ix_max 
          iy0 = mask%iy_min
          iy1 = mask%iy_max

          do iy = iy0, iy1
            do ix = ix0, ix1
              if (mask%mask(ix-ix0+1,iy-iy0+1).eq.1) then  
                
                x1=ix ! +0.5
                y1=iy ! +0.5
         
                s1=0.                  
                do n=1,is

                   select case (point_resp(i))       
                     case(1)
                      s1=s1+scts(ipoint,n)*psf1(energy,j, &
                         x1,y1,x0(n)-xoff(expip(i)),y0(n)-yoff(expip(i)), &
                         xcen(i),ycen(i),sig(n), ecut)
                     case(2)
                      s1=s1+scts(ipoint,n)*psf2(energy,j, &
                         x1,y1,x0(n)-xoff(expip(i)),y0(n)-yoff(expip(i)), &
                         xcen(i),ycen(i),sig(n), ecut)
                     case(3)
                      s1=s1+scts(ipoint,n)*psf3(energy,j, &
                         x1,y1,x0(n)-xoff(expip(i)),y0(n)-yoff(expip(i)), &
                         xcen(i),ycen(i),sig(n), ecut)
                     case(4)
                      s1=s1+scts(ipoint,n)*psf4(energy,j, &
                         x1,y1,x0(n)-xoff(expip(i)),y0(n)-yoff(expip(i)), &
                         xcen(i),ycen(i),sig(n), ecut)
                     case default
                       call error("WrongPSF","Wrong PSF number") 
                  end select               
                end do

!                if (expfla) then
!                  s1=s1*exp_image(ipoint,ix,iy)/exp_center(ipoint)
!                end if
                s1=s1*fudge+bg_image(ipoint,ix,iy) 
                yfit1(l)=s1
                s2=s2+s1
                l=l+1
    
              end if

            end do
          end do
 
          yfit2(ipoint)=s2   
          s3=s3+s2

        end do

        yfit3=yfit3+s3
     
      else

      end if

    end do          

  end subroutine model

end module emlfit



  


