CDISCLAIMER
C ***********************************************************************
C BECAUSE  EXSAS  IS FREE OF CHARGE, THERE IS NO WARRANTY FOR THE PROGRAM,
C TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED
C IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM
C "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,
C INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
C AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY
C AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE
C DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR
C CORRECTION.
C 
C IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
C WILL  MPE  BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL,
C INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY
C TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA
C BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES
C OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN
C IT HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
C 
C Garching, 27 April 1998
C ***********************************************************************
CDISCLAIMER-END
C @(#)equgal.for	1.1 (MPE-Garching) 3/16/94 16:27:36
      subroutine EQUGAL(DJD,LDIR,DLOLD,DBOLD,DLNEW,DBNEW,IERR)
C
CC    Converts coordinates between equatorial and new galactic system
C
C************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:02
C
CA  author : KOD        date: 19-JUL-1989 19:45
CU  update : KOD        date: 20-MAR-1990 14:56
CU  update : EXSAS              date: 03-AUG-1993 15:47
C
CT  status: tested
C
C   general description 
CG  Subroutine to convert coordinates between the equatorial and the
CG  new galactic system (LII,BII).
CG  Parameters extracted from:
CG     F.C.Bertiau, 'Programmes for pocket calculators HP-67
CG                   and HP-97 in the field of theoretical and
CG                   observational astronomy',
CG                   1977 Leuven University Press, 'ASTR. COOR. 2'
C
C   call_var.          type I/O description 
CP  DJD                 R8  I   Julian Day for the conversion
CP
CP  LDIR                L4  I   transformation direction:
CP                                LDIR = .true.  : equ --> gal II
CP                                LDIR = .false. : equ <-- gal II
CP
CP  DLOLD               R8  I   old longitude  (radians)
CP  DBOLD               R8  I   old latitude   (radians)
CP
CP  DLNEW               R8    O new longitude  (radians)
CP  DBNEW               R8    O new latitude   (radians)
CP
CP  IERR                I4    O = 0 no error
CP                              = 1 warning
CP                              = 2 error
CP                              = 3 severe error
C
C   include_block name          description 
CI  RCOMMON:CGENL.CMN          general parameter common block
CI  RCOMMON:IGENL.INC         parameter block with trig. constants
C
C   routines_called    type     description 
CR  EULER1              SR      general coordinate transformation
CR  WRERR               SR      formats error code in standard form
CR  ERRSTC              SR      continue error stack for one level
CR  HFLAG               R4      output flag handling routine
CR  WRFLAG              SR      writes to FLAG output stream
C
C   extensions/system calls     description 
CX                              
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
      INCLUDE 'RCOMMON:CGENL.CMN'
      INCLUDE 'RCOMMON:IGENL.INC'
C
      DOUBLE PRECISION
     &        DJD, DLOLD, DBOLD, DLNEW, DBNEW, D11, D21, D31,
     &              D12, D22, D32, D1, D2, D3, DJD50, DPHI1, DTHETA1,
     &              DPHI2, DTHETA2, DT
      REAL        HFLAG
      INTEGER     IERR ,IOS, ISTAT
      LOGICAL     LDIR
      CHARACTER*8   RTNAME
C
C     julian day for 1950.01.01 / 00:00:00
      PARAMETER ( DJD50 = 2433282.5D0 )
C
      PARAMETER ( D11 = 4.92619181375D0 )
      PARAMETER ( D21 = 1.09257611174D0 )
      PARAMETER ( D31 = 5.70722665400D0 )
C
      PARAMETER ( D12 = 5.82362268000D-7 )
      PARAMETER ( D22 = 2.59857718339D-7 )
      PARAMETER ( D32 = 0.64223753852D-7 )
C
      SAVE
      DATA          RTNAME /'EQUGAL'/
C
      IERR = 0
C
      if (DLOLD.lt.0.D0 .or. DLOLD.GT.D2PI) then
        IERR = 1
        call WRERR(RTNAME,IERR,'Warning: unusual value for DLOLD')
      endif
C
      if (dabs(DBOLD).gt.DPID2) then
        IERR = max(IERR,2)
        call WRERR(RTNAME,IERR,'Error in input parameter DBOLD')
      endif
C
      if (IERR.lt.2) then
C
        DT = DJD - DJD50
C
        ISTAT = 0
        if (dabs(DT).gt.5.D4) ISTAT = 1
        if (dabs(DT).gt.5.D6) ISTAT = 2
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call WRERR(RTNAME,IERR,'Error in input parameter DJD')
        endif
C
      endif
C
      if  (IERR.lt.2)  then
C
        D1 = D11 + D12 * DT
        D2 = D21 + D22 * DT
        D3 = D31 + D32 * DT
C
        DPHI1   = DLOLD
        DTHETA1 = DPID2 - DBOLD
C
        if (LDIR) then
          call EULER1(D1,D2,D3,DPHI1,DTHETA1,DPHI2,DTHETA2,ISTAT)
        else
          call EULER1(-D3,-D2,-D1,DPHI1,DTHETA1,DPHI2,DTHETA2,ISTAT)
        endif
C
        if (ISTAT.gt.1) then
          IERR = max(IERR,ISTAT)
          call ERRSTC(RTNAME,10)
        else
          DLNEW = DPHI2
          DBNEW = DPID2 - DTHETA2
        endif
C
      endif
C
C
      RETURN
      END
